@@ -44,8 +44,12 @@ static PyMethodDef yajlencoder_methods[] = {
 };
 
 static PyTypeObject YajlDecoderType = {
+#ifdef IS_PYTHON3
+    PyVarObject_HEAD_INIT(NULL, 0)
+#else
     PyObject_HEAD_INIT(NULL)
     0,                         /*ob_size*/
+#endif
     "yajl.YajlDecoder",        /*tp_name*/
     sizeof(_YajlDecoder),      /*tp_basicsize*/
     0,                         /*tp_itemsize*/
@@ -85,8 +89,12 @@ static PyTypeObject YajlDecoderType = {
 }; 
 
 static PyTypeObject YajlEncoderType = {
+#ifdef IS_PYTHON3
+    PyVarObject_HEAD_INIT(NULL, 0)
+#else
     PyObject_HEAD_INIT(NULL)
     0,                         /*ob_size*/
+#endif
     "yajl.YajlEncoder",        /*tp_name*/
     sizeof(_YajlEncoder),      /*tp_basicsize*/
     0,                         /*tp_itemsize*/
@@ -146,13 +154,54 @@ static PyObject *py_loads(PYARGS)
     return result;
 }
 
+static unsigned int __config_gen_config(PyObject *indent, yajl_gen_config *config)
+{
+    long indentLevel = -1;
+    char *spaces = NULL; /* XXX: LEAKS */
+
+    if ( (indent) && (indent != Py_None) && (!PyLong_Check(indent)) 
+#ifndef IS_PYTHON3
+            && (!PyInt_Check(indent))
+#endif   
+    ) {
+        PyErr_SetObject(PyExc_TypeError,
+                PyUnicode_FromString("`indent` must be int or None"));
+        return 0;
+    }
+
+    if (indent != Py_None) {
+        indentLevel = PyLong_AsLong(indent);
+
+        if (indentLevel >= 0) {
+            config->beautify = 1;
+            if (indentLevel == 0) {
+                config->indentString = "";
+            }
+            else { 
+                spaces = (char *)(malloc(sizeof(char) * (indentLevel + 1)));
+                memset((void *)(spaces), (int)' ', indentLevel);
+                spaces[indentLevel] = '\0';
+                config->indentString = spaces;
+            }
+        }
+    }
+    return 1;
+}
+
 static PyObject *py_dumps(PYARGS)
 {
     PyObject *encoder = NULL;
     PyObject *obj = NULL;
     PyObject *result = NULL;
+    PyObject *indent = NULL;
+    yajl_gen_config config = { 0, NULL };
+    static char *kwlist[] = {"object", "indent", NULL};
 
-    if (!PyArg_ParseTuple(args, "O", &obj)) {
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O|O", kwlist, &obj, &indent)) {
+        return NULL;
+    }
+
+    if (!__config_gen_config(indent, &config)) {
         return NULL;
     }
 
@@ -161,7 +210,7 @@ static PyObject *py_dumps(PYARGS)
         return NULL;
     }
 
-    result = _internal_encode((_YajlEncoder *)encoder, obj);
+    result = _internal_encode((_YajlEncoder *)encoder, obj, config);
     Py_XDECREF(encoder);
     return result;
 }
@@ -173,13 +222,14 @@ static PyObject *_internal_stream_load(PyObject *args, unsigned int blocking)
     PyObject *stream = NULL;
     PyObject *buffer = NULL;
     PyObject *result = NULL;
+    PyObject *bufferstring = NULL;
 
     if (!PyArg_ParseTuple(args, "O", &stream)) {
         goto bad_type;
     }
 
     if (__read == NULL) {
-        __read = PyString_FromString("read");
+        __read = PyUnicode_FromString("read");
     }
 
     if (!PyObject_HasAttr(stream, __read)) {
@@ -191,19 +241,31 @@ static PyObject *_internal_stream_load(PyObject *args, unsigned int blocking)
     if (!buffer)
         return NULL;
 
+#ifdef IS_PYTHON3
+    bufferstring = PyUnicode_AsUTF8String(buffer);
+    if (!bufferstring)
+        return NULL;
+#endif
+
     decoder = PyObject_Call((PyObject *)(&YajlDecoderType), NULL, NULL);
     if (decoder == NULL) {
         return NULL;
     }
 
+#ifdef IS_PYTHON3
+    result = _internal_decode((_YajlDecoder *)decoder, PyBytes_AsString(bufferstring),
+                PyBytes_Size(bufferstring));
+    Py_XDECREF(bufferstring);
+#else
     result = _internal_decode((_YajlDecoder *)decoder, PyString_AsString(buffer),
                   PyString_Size(buffer));
+#endif
     Py_XDECREF(decoder);
     Py_XDECREF(buffer);
     return result;
 
 bad_type:
-    PyErr_SetObject(PyExc_TypeError, PyString_FromString("Must pass a single stream object"));
+    PyErr_SetObject(PyExc_TypeError, PyUnicode_FromString("Must pass a single stream object"));
     return NULL;
 }
 
@@ -217,19 +279,14 @@ static PyObject *py_iterload(PYARGS)
 }
 
 static PyObject *__write = NULL;
-static PyObject *_internal_stream_dump(PyObject *args, unsigned int blocking)
+static PyObject *_internal_stream_dump(PyObject *object, PyObject *stream, unsigned int blocking, 
+            yajl_gen_config config)
 {
     PyObject *encoder = NULL;
-    PyObject *stream = NULL;
     PyObject *buffer = NULL;
-    PyObject *object = NULL;
-
-    if (!PyArg_ParseTuple(args, "OO", &object, &stream)) {
-        goto bad_type;
-    }
 
     if (__write == NULL) {
-        __write = PyString_FromString("write");
+        __write = PyUnicode_FromString("write");
     }
 
     if (!PyObject_HasAttr(stream, __write)) {
@@ -241,24 +298,43 @@ static PyObject *_internal_stream_dump(PyObject *args, unsigned int blocking)
         return NULL;
     }
 
-    buffer = _internal_encode((_YajlEncoder *)encoder, object);
+    buffer = _internal_encode((_YajlEncoder *)encoder, object, config);
     PyObject_CallMethodObjArgs(stream, __write, buffer, NULL);
     Py_XDECREF(encoder);
     return Py_True;
 
 bad_type:
-    PyErr_SetObject(PyExc_TypeError, PyString_FromString("Must pass a stream object"));
+    PyErr_SetObject(PyExc_TypeError, PyUnicode_FromString("Must pass a stream object"));
     return NULL;
 }
+
 static PyObject *py_dump(PYARGS)
 {
-    return _internal_stream_dump(args, 0);
+    PyObject *object = NULL;
+    PyObject *indent = NULL;
+    PyObject *stream = NULL;
+    yajl_gen_config config = { 0, NULL };
+    static char *kwlist[] = {"object", "stream", "indent", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwargs, "OO|O", kwlist, &object, &stream, &indent)) {
+        return NULL;
+    }
+    if (!__config_gen_config(indent, &config)) {
+        return NULL;
+    }
+    return _internal_stream_dump(object, stream, 0, config);
 }
 
 static struct PyMethodDef yajl_methods[] = {
-    {"dumps", (PyCFunction)(py_dumps), METH_VARARGS, 
-"yajl.dumps(obj)\n\n\
-Returns an encoded JSON string of the specified `obj`"},
+    {"dumps", (PyCFunctionWithKeywords)(py_dumps), METH_VARARGS | METH_KEYWORDS,
+"yajl.dumps(obj [, indent=None])\n\n\
+Returns an encoded JSON string of the specified `obj`\n\
+\n\
+If `indent` is a non-negative integer, then JSON array elements \n\
+and object members will be pretty-printed with that indent level. \n\
+An indent level of 0 will only insert newlines. None (the default) \n\
+selects the most compact representation.\n\
+"},
     {"loads", (PyCFunction)(py_loads), METH_VARARGS, 
 "yajl.loads(string)\n\n\
 Returns a decoded object based on the given JSON `string`"},
@@ -266,10 +342,16 @@ Returns a decoded object based on the given JSON `string`"},
 "yajl.load(fp)\n\n\
 Returns a decoded object based on the JSON read from the `fp` stream-like\n\
 object; *Note:* It is expected that `fp` supports the `read()` method"},
-    {"dump", (PyCFunction)(py_dump), METH_VARARGS, 
-"yajl.dump(obj, fp)\n\n\
+    {"dump", (PyCFunctionWithKeywords)(py_dump), METH_VARARGS | METH_KEYWORDS,
+"yajl.dump(obj, fp [, indent=None])\n\n\
 Encodes the given `obj` and writes it to the `fp` stream-like object. \n\
-*Note*: It is expected that `fp` supports the `write()` method"},
+*Note*: It is expected that `fp` supports the `write()` method\n\
+\n\
+If `indent` is a non-negative integer, then JSON array elements \n\
+and object members will be pretty-printed with that indent level. \n\
+An indent level of 0 will only insert newlines. None (the default) \n\
+selects the most compact representation.\n\
+"},
     /*
      {"iterload", (PyCFunction)(py_iterload), METH_VARARGS, NULL},
      */
@@ -277,11 +359,16 @@ Encodes the given `obj` and writes it to the `fp` stream-like object. \n\
 };
 
 
-
+#ifdef IS_PYTHON3
+static struct PyModuleDef yajlmodule = {
+    PyModuleDef_HEAD_INIT,
+    "yajl",
+#else
 PyMODINIT_FUNC inityajl(void)
 {
 
     PyObject *module = Py_InitModule3("yajl", yajl_methods, 
+#endif
 "Providing a pythonic interface to the yajl (Yet Another JSON Library) parser\n\n\
 The interface is similar to that of simplejson or jsonlib providing a consistent syntax for JSON\n\
 encoding and decoding. Unlike simplejson or jsonlib, yajl is **fast** :)\n\n\
@@ -292,20 +379,43 @@ yajl.loads():\t\t502.4572ms\n\
 \n\
 json.dumps():\t\t7760.6348ms\n\
 simplejson.dumps():\t930.9748ms\n\
-yajl.dumps():\t\t681.0221ms");
+yajl.dumps():\t\t681.0221ms"
+#ifdef IS_PYTHON3
+    , -1, yajl_methods, NULL, NULL, NULL, NULL
+};
+
+PyMODINIT_FUNC PyInit_yajl(void) 
+{
+    PyObject *module = PyModule_Create(&yajlmodule);
+#else
+);
+#endif
 
     YajlDecoderType.tp_new = PyType_GenericNew;
-    if (PyType_Ready(&YajlDecoderType) < 0)
-        return;
+    if (PyType_Ready(&YajlDecoderType) < 0) {
+        goto bad_exit;
+    }
 
     Py_INCREF(&YajlDecoderType);
     PyModule_AddObject(module, "Decoder", (PyObject *)(&YajlDecoderType));
 
     YajlEncoderType.tp_new = PyType_GenericNew;
-    if (PyType_Ready(&YajlEncoderType) < 0)
-        return;
+    if (PyType_Ready(&YajlEncoderType) < 0) {
+        goto bad_exit;
+    }
 
     Py_INCREF(&YajlEncoderType);
     PyModule_AddObject(module, "Encoder", (PyObject *)(&YajlEncoderType));
+
+#ifdef IS_PYTHON3
+    return module;
+#endif
+
+bad_exit:
+#ifdef IS_PYTHON3
+    return NULL;
+#else
+    return;
+#endif
 }
 
